//+---------------------------------------------------------------------+
//| MT4 Indicator:                                P4L CandleTime.mq4    |
//| Revised: v1_2 2009-11-13 by pips4life, a user at forexfactory.com   |
//|                                                                     |
//| Previous names: b-Clock.mq4, CandleTime.mq4...                      |
//|                                     Core time code by Nick Bilak    |
//|        http://metatrader.50webs.com/         beluck[at]gmail.com    |
//|                                  modified by adoleh2000 and dwt5    | 
//|                                                                     | 
//|                                       |
//+---------------------------------------------------------------------+
//
// Instructions:
// Copy this file to to your MT4 indicators folder:
// On XP, usually:  C:\Program Files\__your_MT4_broker__\experts\indicators\
//
// Review the "extern" variable settings below. Change as desired, then restart MT4 or do "Compile" in MetaEditor.
// Open a chart and add this indicator.

// Version History: 
// v1_2 2009-11-13 by pips4life, a user at forexfactory.com
//   Added new "d" for "days" variable. Times on Weekly/Monthly charts now
//     limit "h" (hours) to under 24. New display is:  d_h:mm:yy  (only if d>0).
//     Whereas the old version displayed "387:15:27", the new version is now "16_3:15:27"
//   Added new AutoTimeShiftAdjust feature to support Symbol() names generated by
//     "P4L PeriodCon.mq4".   That other program can generate offline charts with a
//     timeshift (i.e. offset).   When "true", this indicator looks for Symbol() names 
//     using the format "EURUSD+1H" or "EURUSD-2H" where "+1H", "-2H", (or +/- any 
//     number of M/H/D/W/MN) is used as an adjustment to get the correct secondsleft.
// v1_1 2008-10-02 by pips4life, a user at forexfactory.com
//   For H4 and above, display #hours:MM:SS left for the bar. W1 and MN
//   bar times are not accurate though because the weekend hours are included, and
//   Period() does not report an accurate number of seconds/week or seconds/month.
// v1_0 2008-09-24 by pips4life, a user at forexfactory.com
//   A rewrite of CandleTime.mq4 to improve display of MM:SS



//#property copyright "Copyright  2005, Nick Bilak" // previous author
//#property link      "http://metatrader.50webs.com/"
#property copyright "pips4life"
#property link      "Search http://forexfactory.com for P4L CandleTime.mq4"
#property indicator_chart_window

extern color  TextColor = Gold;
extern int    FontSize  = 9;
extern string FontName = "Verdana";
extern bool   DisplayTimeByTheBar = true ;
extern bool   DisplayTimeComment  = false;
extern bool   AutoTimeShiftAdjust = true; // Necessary when used with "P4L PeriodCon.mq4". It adjusts for Symbol() names like: "EURUSD+1H"
                                          // However, change to "false" if your Symbol() name DOES contain a natural "-" or "+" sign.

bool Debug = false;
datetime timeshiftsec = 0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
  {
   if (AutoTimeShiftAdjust)
     {
      // The following detects symbol names like "EURUSD+1H,Daily" as generated by "P4L PeriodCon.mq4"
      // The "+1H" string is a timeshift (i.e. offset) that must be factored in.
      // There are extremely few regular "Symbol()" names that have a true "+" or "-" in
      // them, so this method should be sufficient. However, if there's a problem, turn off AutoTimeShiftAdjust.
   
      string symbolOffset;
      int plus = StringFind(Symbol(),"+",0);
      int minus = StringFind(Symbol(),"-",0);
      
      if ( plus > 0) symbolOffset = StringSubstr(Symbol(),plus);      // The "+" is kept if "plus", dropped if "plus+1".
      else if ( minus > 0) symbolOffset = StringSubstr(Symbol(),minus); // The "-" stays with the string.
      
      if (Debug) Print("Symbol(): ",Symbol()," plus: ",plus," minus: ",minus," symbolOffset: ",symbolOffset);
      
      if (StringLen(symbolOffset) <= 1) return(0); // str is at least 2 chars, normally would be min, e.g. "+1M"
      
      if (StringFind(symbolOffset,"MN",0) > 0)
        timeshiftsec = 60 * PERIOD_MN1 * StrToInteger(StringSubstr(symbolOffset,0,StringFind(symbolOffset,"MN",0)));
      else if (StringFind(symbolOffset,"W",0) > 0) 
        timeshiftsec = 60 * PERIOD_W1 * StrToInteger(StringSubstr(symbolOffset,0,StringFind(symbolOffset,"W",0)));
      else if (StringFind(symbolOffset,"D",0) > 0) 
        timeshiftsec = 60 * PERIOD_D1 * StrToInteger(StringSubstr(symbolOffset,0,StringFind(symbolOffset,"D",0)));
      else if (StringFind(symbolOffset,"H",0) > 0) 
        timeshiftsec = 60 * PERIOD_H1 * StrToInteger(StringSubstr(symbolOffset,0,StringFind(symbolOffset,"H",0)));
      else if (StringFind(symbolOffset,"M",0) > 0) 
        timeshiftsec = 60 * StrToInteger(StringSubstr(symbolOffset,0,StringFind(symbolOffset,"M",0)));
      //else
      //  timeshiftsec = 60 * StrToInteger(StringSubstr(symbolOffset,0)); //Probably not a good idea to do this.
      
      if (Debug && StringLen(symbolOffset) > 0) Print("timeshiftsec: ",timeshiftsec);
     }
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() 
  {
   ObjectDelete("time");
   return(0);
  } 
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   static datetime last_timecurrent;
   if (TimeCurrent() - last_timecurrent == 0) return; // Little value in processing more than 1 tick per second, but the text position may lag slightly.
   last_timecurrent = TimeCurrent();
   
   int secondsleft = Time[0]+Period()*60-TimeCurrent()-timeshiftsec;
   
   //Print("Time[0]=",Time[0], "  Period()=",Period(),"  TimeCurrent()=",TimeCurrent(),"  secondsleft=",secondsleft);
   int d,h,m,s;
   s=secondsleft%60;
   m=((secondsleft-s)/60)%60;
   //h=(secondsleft-s-m*60)/3600;
   h=((secondsleft-s-m*60)/3600)%24;
   d=(secondsleft-s-m*60-h*3600)/86400; //1day=86400sec
   
   if( DisplayTimeComment) 
     {
      if (d!=0) Comment( d + " days " + h + " hours " + m + " minutes " + s + " seconds left to bar end");
      else if (h!=0) Comment( h + " hours " + m + " minutes " + s + " seconds left to bar end");
      else Comment( m + " minutes " + s + " seconds left to bar end");
     }
	
	ObjectDelete("time"); // The only reason to delete every time, AFAIK, is to unselect the object in case it was selected by accident.
   
   string displaystr;
   // Note, the prefix of spaces below is intentional and necessary to keep the top-middle-anchored text off of Bar[0]!
   if (secondsleft >= 0 && h==0 && d==0) 
     {
      displaystr = StringConcatenate("                      <-- ",StringSubstr(TimeToStr(secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   else if (secondsleft >= 0 && d>0) 
     {
      displaystr = StringConcatenate("                              <-- ",d,"_",h,":",StringSubstr(TimeToStr(secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   else if (secondsleft >= 0 && h>0) 
     {
      displaystr = StringConcatenate("                            <-- ",h,":",StringSubstr(TimeToStr(secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   else if (h==0 && d==0)
     {// When 1 bar is complete, before a new bar is formed the old version displayed hard-to-read negative values. The new is very explicit:
      // FYI, max of ~62 characters
      displaystr = StringConcatenate("                                        <-- WAIT4BAR:-",StringSubstr(TimeToStr(-1*secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   else if (d==0) //h<0
     {// When 1 bar is complete, before a new bar is formed the old version displayed hard-to-read negative values. The new is very explicit:
      // FYI, max of ~62 characters
      displaystr = StringConcatenate("                                               <-- WAIT4BAR:-",-1*h,":",StringSubstr(TimeToStr(-1*secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   else //d<0 AND h<0
     {// When 1 bar is complete, before a new bar is formed the old version displayed hard-to-read negative values. The new is very explicit:
      // FYI, max of ~62 characters
      displaystr = StringConcatenate("                                                 <-- WAIT4BAR:-",-1*d,"_",-1*h,":",StringSubstr(TimeToStr(-1*secondsleft,TIME_MINUTES|TIME_SECONDS),3));
     }
   
   
   if(DisplayTimeByTheBar && ObjectFind("time") < 0) // -1 is < 0 if object does not exist.  FYI, if it was just deleted, might not need to "ObjectFind" it first.
     {
      ObjectCreate("time", OBJ_TEXT, 0, Time[0], Close[0]+ 0.0005);
      ObjectSetText("time", displaystr, FontSize, FontName, TextColor);
     }
   else if (DisplayTimeByTheBar && ObjectFind("time") == 0)
     {
      ObjectMove("time", 0, Time[0], Close[0]+0.0005);
      ObjectSetText("time", displaystr, FontSize, FontName, TextColor);
     }


   return(0);
  }
//+------------------------------------------------------------------+


